/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/alternatives.h
* Description:  Alternatives in a rule {X, Y, ..., Z}, not in an AVM.
* Author:       Brett Kessler
* Created:      Wed Apr 15 17:06:26 1992
* Modified:     Sun May  3 16:42:18 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(ALTERNATIVES_H)
#define ALTERNATIVES_H

#include "dlist.h"
#include "match.h"

/*
 * Alternatives
 *
 * Alternatives are a list of items, any one of which may make the 
 * entire expression succeed if it succeeds.  items is an array of
 * nItems Items.
 */
typedef struct alternatives {
  int nItems;
  void** /* Item* */ items;
}* Alternatives;

/*
 * Alternatives_Match
 *
 * Attempts to match the alternatives object in the pattern matching
 * context "facts".  If any one of the alternatives leads to a successful
 * match, returns immediately with that success.
 */
extern void
Alternatives_Match(Alternatives alts, Match_Facts facts);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
