/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/attribute.h
* Description:  A lexical attribute.
* Author:       Brett Kessler
* Created:      Sun Apr 26 21:22:53 1992
* Modified:     Sun May  3 16:42:16 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(ATTRIBUTE_H)
#define ATTRIBUTE_H

#include "sign.h"
#include "terminal.h"

/*
 * Attribute
 *
 * A lexical attribute (represented by its numeric ID) and the sign, which
 * must be a constant Sign, telling whether the attribute is to be found
 * (Sign_PLUS) or not to be found (Sign_MINUS).
 */
typedef struct attribute {
  Sign direction;
  int id;
}* Attribute;

/*
 * Attribute_Create
 *
 * Creates an Attribute, given its sign and numeric ID.
 */
extern Attribute
Attribute_Create(Sign direction, int id);

/*
 * Attribute_Match
 *
 * Tells whether the set of lexical properties (associated with the current
 * word in the string being matched) match the Attribute specification.
 * That is, if the Attribute is Sign_PLUS, succeeds if the word has the
 * relevant attribute; if Sign_MINUS, succeeds only if the word does not
 * have the attribute.
 */
extern Boolean
Attribute_Match(Attribute attribute, Terminal_Properties props);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
