/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/attributes.h
* Description:  A set of attributes for a word (in a rule).
* Author:       Brett Kessler
* Created:      Sun Apr 26 21:28:53 1992
* Modified:     Sun May  3 16:42:15 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(ATTRIBUTES_H)
#define ATTRIBUTES_H

#include "attribute.h"
#include "header.h"
#include "terminal.h"

/*
 * Attributes
 *
 * An array (with set semantics) of Attribute.
 */
typedef struct attributes {
  int nAttributes;
  Attribute* array;
}* Attributes;

/*
 * Attributes_Create
 *
 * Creates an Attributes object from the enumerated Attribute's.  The ...
 * stands for a series of Attribute arguments; nAttributes counts them.
 */
extern Attributes
Attributes_Create(int nAttributes, ...);

/*
 * Attributes_Match
 *
 * Succeeds if the word-boundary terminal found in the string being matched
 * agrees with the set of lexical attributes.
 */
extern Boolean
Attributes_Match(Attributes attributes, Terminal wb);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
