
/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/avm.c
* Description:  Implements avm.h
* Author:       Brett Kessler
* Created:      Wed Apr 15 20:38:26 1992
* Modified:     Sun May  3 16:42:12 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#include "avm.h"
#include "dlist.h"
#include "header.h"
#include "Malloc.h"
#include "match.h"
#include "sign.h"
#include <stdarg.h>

AVM
AVM_Create(int nFeatures, ...)
{
  AVM a;
  va_list args;
  int f;
  a = Malloc_Struct(avm);
  a->nFeatures = nFeatures;
  a->features = Malloc(nFeatures * sizeof (Feature));
  va_start(args, nFeatures);
  for (f = 0; f < nFeatures; f++) a->features[f] = va_arg(args, Feature);
  va_end(args);
  return a;
}

void
AVM_Match(AVM avm, Match_Facts facts)
{
  int f;
  Boolean matches = TRUE;
  DList_Cons currentPos;
  Sign_VarsVector origVars;
  int v;
  Terminal currentTerm;
  currentPos = facts->in.text;
  currentTerm = currentPos->datum;
  if (currentTerm->type != Terminal_segmentType) return;
  for (v = 0; v < 4; v++) origVars[v] = facts->in.vars[v];
  for (f = 0; f < avm->nFeatures; f++)
    if (!Feature_Match(avm->features[f], 
		       currentTerm->u.segment,
		       facts->in.vars)) {
      matches = FALSE;
      break;
    }
  if (matches) {
    facts->in.text = currentPos->next;
    Match_Continue(facts);
    if (facts->out.success) return;
  }
  facts->in.text = currentPos;
  for (v = 0; v < Sign_NSIGNS; v++) facts->in.vars[v] = origVars[v];
}

DList_Cons
AVM_InsertAfter(AVM avm,
		Terminal_String string,
		DList_Cons afterCons,
		Sign_VarsVector variables,
		Segment deletedSegment)
{
  int f;
  for (f = 0; f < avm->nFeatures; f++)
    deletedSegment = Feature_Add(avm->features[f], deletedSegment, variables);
  return Terminal_InsertSegment(string, deletedSegment, afterCons);
}

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
