/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/avm.h
* Description:  AVMs representing phonetic feature bundles in rules.
* Author:       Brett Kessler
* Created:      Wed Apr 15 17:11:36 199
* Modified:     Sun May  3 16:42:14 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(AVM_H)
#define AVM_H

#include "dlist.h"
#include "feature.h"
#include "header.h"
#include "match.h"
#include <stdio.h>

/*
 * AVM
 *
 * An AVM is a set of Feature, here represented as an array of nFeatures
 * objects.
 */
typedef struct avm {
  int nFeatures;
  Feature* features;
}* AVM;

/*
 * AVM_Create
 *
 * Creates an AVM.  The arguments are nFeatures, which tells how many
 * features are coming, then those Feature arguments in turn.
 */
extern AVM
AVM_Create(int nFeatures, ...);

/*
 * AVM_Match
 *
 * Attempts to match the AVM, and the rest of the current agenda, against
 * the current state of the pattern matching.  The current item must be a
 * segment and match all of the subfeatures.
 */
extern void
AVM_Match(AVM avm, Match_Facts facts);

/*
 * AVM_InsertAfter
 * 
 * Inserts the AVM into the data string, as part of the structural
 * change in a rule.  Since an AVM change affects only the features explicitly
 * listed in the AVM, the segment that was deleted (or to which the AVM
 * was indexed) is also passed in.  The variables argument allows substitution
 * of alpha variables.
 */
extern DList_Cons
AVM_InsertAfter(AVM avm,
		Terminal_String string,
		DList_Cons afterCons,
		Sign_VarsVector variables,
		Segment deletedSegment);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
