/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/derive.c
* Description:  Main module for derive: test out the rules
* Author:       Brett Kessler
* Created:      Wed Apr 15 15:01:17 1992
* Modified:     Sun May  3 18:11:42 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#include "header.h"
#include "ruleSystem.h"
#include "readWordsYacc.h"
#include <string.h>
#include <unistd.h>  //getopt

typedef enum characterCode {
  noCharacterCode,
  asciiCharacterCode,
  latexCharacterCode
} CharacterCode;

/*
 * characterCode
 *
 * Tells whether to print the derivations in ASCII or in LaTeX.
 * Can be rest by comman dline option -c
 */
static CharacterCode
characterCode = asciiCharacterCode;

#define EQUAL 0

static CharacterCode
ReadCharacterCode(char* name)
{
  if (strcmp(name, "ascii") == EQUAL) return asciiCharacterCode;
  else if (strcmp(name, "latex") == EQUAL) return latexCharacterCode;
  else return noCharacterCode;
}

static void
PrintOptions(void) __attribute__ ((noreturn));

/* Explain command line options, then exit programme. */
static void
PrintOptions(void)
{
  (void) 
    printf("derive [-c code]\n"
	   "  Reads test suite on input, generates derivations. Options:\n"
	   "  -c ascii   (Default) Print in ASCII code.\n"
	   "  -c latex   Print LaTeX output.\n");
  exit(2);
}

/* Parse command line options. */
static void
ParseOptions(int argc, char* argv[])
{
  Boolean error = FALSE;
  if (argc == 1) characterCode = asciiCharacterCode;
  else {
    int letter;
    while ((letter = getopt(argc, argv, "c:h")) != EOF) switch (letter) {
    case 'c':
      characterCode = ReadCharacterCode(optarg); 
      if (characterCode == noCharacterCode) error = TRUE;
      break;
    case 'h': PrintOptions(); 
    case '?': error = TRUE;
    }
    if (error) PrintOptions();
  }
}

int
main(int argc, char* argv[])
{
  ParseOptions(argc, argv);
  RuleSystem_Initialize();
  ReadWordsYacc_Parse((characterCode == latexCharacterCode));
  return 0;
}

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
