/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/dlist.c
* Description:  Implements dlist.h
* Author:       Brett Kessler
* Created:      Sun Apr 19 10:08:50 1992
* Modified:     Sun May  3 18:11:41 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#include "dlist.h"
#include <stdio.h>
#include <stdlib.h>

void
DList_Init(DList list)
{
  list->count = 0;
  list->head = list->tail = NULL;
  list->consResource = NULL;
}

DList
DList_Create(void)
{
  DList list;
  list = Malloc_Struct(dlist);
  DList_Init(list);
  return list;
}

static DList_Cons
Add(DList* list, void* datum)
{
  DList_Cons cons;
  if (*list == NULL) {
    *list = DList_Create();
    if (*list == NULL) return NULL;
  }
  cons =
    ((*list)->consResource == NULL) ?
      Malloc(sizeof (struct DList_cons)) 
    : Resource_Get((*list)->consResource);
  if (cons == NULL) return NULL;
  cons->datum = datum;
  cons->next = NULL;
  if ((*list)->tail == NULL) (*list)->head = cons;
  else ((*list)->tail)->next = cons;
  cons->prev = (*list)->tail;
  (*list)->tail = cons;
  (*list)->count++;
  return cons;
}

DList_Cons
DList_AddCons(DList list, void* datum)
{
  return Add(&list, datum);
}

DList
DList_Add(DList list, void* datum)
{
  DList_Cons cons;
  cons = Add(&list, datum);
  if (cons == NULL) return NULL;
  return list;
}

Resource
DList_CreateCons(unsigned int consPerBlock)
{
  return Resource_Create(sizeof (struct DList_cons), consPerBlock);
}

Boolean
DList_Equal(DList list1, DList list2, DList_Comparer equals)
{
  DList_Cons c1;
  DList_Cons c2;
  if (list1 == NULL) return (list2 == NULL);
  if (list2 == NULL) return FALSE;
  if (list1->count != list2->count) return FALSE;
  for (c1 = list1->head, c2 = list2->head;
       c1 != NULL;
       c1 = c1->next, c2 = c2->next) {
    if (c1 == NULL || c2 == NULL) {
      (void) fprintf(stderr, "DList_Equal c1 = %X, c2 = %X\n", 
		     (int) c1, (int) c2);
      exit(1);
    }
    if (!equals(c1->datum, c2->datum)) return FALSE;
  }
  return TRUE;
}

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
