/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/feature.h
* Description:  Interface file.
* Author:       Brett Kessler
* Created:      Sun Apr 26 08:24:41 1992
* Modified:     Sun May  3 18:11:38 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(FEATURE_H)
#define FEATURE_H

#include "header.h"
#include "plainSignedFeature.h"
#include "segment.h"
#include "sign.h"
#include <stdio.h>

/*
 * Feature
 *
 * A top-level element in a phonetic feature bundle (AVM).  This is
 * usually a single plain signed feature (e.g, +cons), but can also be
 * more than one of them, representing alternatives (e.g., {+high, -lab}).
 */
typedef struct feature {
  int nPlains;
  PlainSignedFeature* plains;  /* alternatives */
}* Feature;

/*
 * Feature_Create
 *
 * Creates a Feature from plain signed features.  The first argument tells
 * how many such features there are; the following arguments are that 
 * many PlainSignedFeature's.
 */
extern Feature
Feature_Create(int nPlains, ...);

/*
 * Feature_Match
 *
 * Succeeds if the current segment matches the Feature.  If one of
 * the component plain signed features has an alpha variable, the
 * value found is either recorded in vars or compared against the 
 * value found when the variable was encountered earlier in the pattern.
 */
extern Boolean
Feature_Match(Feature feature, Segment segment, Sign_VarsVector vars);

/*
 * Feature_Add
 *
 * Adds the information specified in the Feature to the segment (as part of
 * a structural change).  The alpha variables found in the pattern match
 * are passed in, in case one of those variables is mentioned in the 
 * structural change.
 */
extern Segment
Feature_Add(Feature feature, Segment segment, Sign_VarsVector variables);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
