/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/features.c
* Description:  Implements features.h
* Author:       Brett Kessler
* Created:      Mon Apr 20 21:39:26 1992
* Modified:     Sun May  3 18:11:36 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#include "features.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../frag/featuresC"

void
Features_PrintFeature(Features_Id feature, FILE* file)
{
  (void) fputs(Features_Names[feature], file);
}

unsigned long int
Features_GetMask(Features_Id feature)
{
  return Features_mask[feature];
}

Features_Id
Features_GetParent(Features_Id feature)
{
  return Features_parent[feature];
}

#define EQUAL 0

unsigned long int
Features_GetSyllMask(void)
{
  static unsigned long int syllMask = 0;
  /* No point in computing this more than once. */
  if (!syllMask) {
    int f;
    for (f = 0; f < Features_COUNT; f++)
      if (strcmp(Features_Names[f], "syll") == EQUAL) {
	syllMask = Features_mask[f];
	break;
      }
    if (!syllMask) {
      (void) printf("Can't find feature named 'syll'");
      exit(1);
    }
  }
  return syllMask;
}

Boolean
Features_IsDescendant(Features_Id child, Features_Id parent)
{
  Features_Id f;
  for (f = child;
       (f != parent && f != Features_NONE); 
       f = Features_GetParent(f));
  return (f == parent);
}

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
