/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/features.h
* Description:  The set of features defined for the system.
* Author:       Brett Kessler
* Created:      Mon Apr 20 21:37:04 1992
* Modified:     Sun May  3 18:11:35 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(FEATURES_H)
#define FEATURES_H

#include "../frag/featuresH"
#include "header.h"
#include <stdio.h>

#define Features_NONE -1

/*
 * Features_Id
 *
 * This is the unique numeric ID for a feature (not the bit mask).
 */
typedef int
Features_Id;

/*
 * Features_Names
 *
 * Given the feature ID, returns the feature name.
 */
extern char*
Features_Names[Features_COUNT + 1];

/*
 * Features_parent
 *
 * Given the feature ID, returns its parent (or Features_NONE).  The
 * children of a composte feature can be found by enumerating this
 * array, noting all features that have the given feature as a parent.
 */
extern Features_Id
Features_parent[Features_COUNT + 1];

/*
 * Features_PrintFeature
 *
 * Prints the name of the feature on the given stream.
 */
extern void
Features_PrintFeature(Features_Id feature, FILE* file);

/*
 * Features_mask
 *
 * Given a feature ID number, returns the bit mask that represents that
 * feature.  This is Features_NONE for composite features.
 */
extern unsigned long int
Features_mask[Features_COUNT + 1];

/*
 * Features_GetMask
 *
 * Given a feature ID number, returns the bit mask that represents that
 * feature.  This is Features_NONE for composite features.
 */
extern unsigned long int
Features_GetMask(Features_Id feature);

/*
 * Features_GetParent
 *
 * Given the feature ID, returns its parent (or Features_NONE).  The
 * children of a composte feature can be found by enumerating this
 * array, noting all features that have the given feature as a parent.
 */
extern Features_Id
Features_GetParent(Features_Id feature);

/*
 * Features_GetSyllMask
 *
 * Returns the bit mask for the syll feature.
 */
extern unsigned long int
Features_GetSyllMask(void);

/*
 * Features_IsDescendant
 *
 * Returns TRUE if the 1st feature is a direct child or ultimate 
 * descendant of the second feature.
 */
extern Boolean
Features_IsDescendant(Features_Id child, Features_Id parent);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
