/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/indexedAvm.h
* Description:  An AVM with an identity index.
* Author:       Brett Kessler
* Created:      Wed Apr 15 17:13:07 1992
* Modified:     Sun May  3 19:35:54 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(INDEXEDAVM_H)
#define INDEXEDAVM_H

#include "avm.h"
#include "match.h"

/*
 * IndexedAVM
 *
 * An AVM (phonetic feature bundle) with an identity index.  The index
 * stored here is a number from 0 to 25, corresponding to the original
 * (in RULES) lower case letters.
 */
typedef struct indexedAVM {
  AVM avm;
  char index;
}* IndexedAVM;

/*
 * IndexedAVM_Create
 *
 * Creates an IndexedAVM from an AVM and the numeric identity index.
 */
extern IndexedAVM
IndexedAVM_Create(AVM avm, char indx);

/*
 * IndexedAVM_Match
 *
 * Attempts to match the AVM against the current token, with the
 * stipulation that AVMs with the same identity index must match the
 * same segment. This is done by checking the state in "facts": the
 * first time an index is encountered, the segment that was matched is
 * recorded; on subsequent encounters, we fail if the current segment
 * does not match what is recorded.
 */
extern void
IndexedAVM_Match(IndexedAVM avm, Match_Facts facts);

/*
 * IndexedAVM_InsertAfter
 * 
 * Inserts the indexed AVM in the data string.  The indexes vector tells
 * what segment the AVM feature change is relative to; the vars vector
 * will be used if the AVM Mentions alpha variables.
 */
extern DList_Cons
IndexedAVM_InsertAfter(IndexedAVM avm,
		       Terminal_String string,
		       DList_Cons afterCons,
		       Match_IndexVector indexes,
		       Sign_VarsVector vars);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
