/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/item.h
* Description:  Top-level pattern (a sequence of factors)
* Author:       Brett Kessler
* Created:      Wed Apr 15 16:56:31 1992
* Modified:     Sun May  3 19:35:52 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(ITEM_H)
#define ITEM_H

#include "dlist.h"
#include "factor.h"
#include "header.h"
#include "match.h"

/*
 * Item
 *
 * A sequence of factors, implemented as an array of length nFactors.
 */
typedef struct item {
  int nFactors;
  Factor* factors;
} *Item;

/*
 * Item_Create
 *
 * Creates an Item from Factor's.  The first argument tells how many
 * factors; then follow that many Factor's.
 */
extern Item
Item_Create(int nFactors, ...);

/*
 * Item_Match
 *
 * Attempts to find the Item at the current point in the match.
 * Pushes all the component factors onto the head of the continuation
 * agenda, then invokes the continuation.
 */
extern void
Item_Match(Item item, Match_Facts facts);

/*
 * Item_MatchOptional
 *
 * Like Item_Match, but first tries the continuation without pushing
 * the factors onto the agenda; that is done only if this fails.
 */
extern void
Item_MatchOptional(Item item, Match_Facts facts);

/*
 * Item_InsertAfter
 *
 * Inserts the item into the token string, as part of the structural
 * change.  The facts are passed in in case the change mentions identity
 * indexes or alpha variables; the deletedSegment is the segment relative to
 * which an unindexed AVM makes its changes.
 */
extern void
Item_InsertAfter(Item new,
		 Terminal_String string,
		 DList_Cons afterCons,
		 Match_Facts facts,
		 Segment deletedSegment);

/*
 * Item_MaximizeOnsets
 *
 * Resyllabifies the token string so as to maximize onsets.  An onset
 * is anything that completely matches the onsetPattern (from ONSETS
 * in the RULES file).  If segments cannot fit into utterance-initial
 * onsets, they are discarded.
 */
extern void
Item_MaximizeOnsets(Item onsetPattern, Terminal_String data);


#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
