/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/plainSignedFeature.h
* Description:  A phonetic feature with a sign: +/- or an alpha variable.
* Author:       Brett Kessler
* Created:      Sun Apr 26 08:24:40 1992
* Modified:     Sun May  3 19:35:49 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(PLAINSIGNEDFEATURE_H)
#define PLAINSIGNEDFEATURE_H

#include "header.h"
#include "segment.h"
#include "sign.h"

/*
 * PlainSignedFeature
 *
 * A phonetic feature (identified by its numeric ID) and a sign.
 */
typedef struct plainSignedFeature {
  Sign sign;
  int featureNumber;
}* PlainSignedFeature;

/*
 * PlainSignedFeature_Create
 *
 * Creates a plain signed feature from a sign and feature ID.
 */
extern PlainSignedFeature
PlainSignedFeature_Create(Sign sign, int featureNumber);

/*
 * PlainSignedFeature_Match
 *
 * Returns TRUE if the segment matches the feature description.  If
 * the feature has a variable, then if this variable hasn't been
 * encountered before, its value is stored in vars and match succeeds.
 * But if it has been encountered, the value previously encountered 
 * must be found here.
 */
extern Boolean
PlainSignedFeature_Match(PlainSignedFeature feature,
			 Segment segment,
			 Sign_VarsVector vars);

/*
 * PlainSignedFeature_Add
 *
 * Adds to the segment (as part of a structural change) the feature described
 * by the first argument, and returns the modified segment. The vector of
 * encountered variables is passed in in case the feature description contains
 * an alpha variable.
 */
extern Segment
PlainSignedFeature_Add(PlainSignedFeature psf, 
		       Segment segment,
		       Sign_VarsVector variables);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
