/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/rule.h
* Description:  A rule.
* Author:       Brett Kessler
* Created:      Wed Apr 15 17:01:54 1992
* Modified:     Sun May  3 20:10:21 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(RULE_H)
#define RULE_H

#include "item.h"
#include "terminal.h"

typedef enum rule_Type {
  Rule_ordinaryType,  /* i.e., explicitly formulated rule */
  Rule_maximizeOnsets  /* resyllabification conforming to ONSETS */
} Rule_Type;

/*
 * Rule
 *
 * An ordinary rule has a name, a structural description (original), and
 * a structural change.  Unlike in rules/, the source and environment are
 * merged into "original", with delimiters surrounding the component
 * that is to be changed.
 */
typedef struct rule {
  Rule_Type type;
  char* name;
  Item original;
  Item change;
}* Rule;

/*
 * Rule_longestName
 *
 * The longest name in the rule system.  Used for columnar alignment.
 */
extern char*
Rule_longestName;

/*
 * Rule_longestNameLen
 *
 * The number of characters in the longest name in the rule system.  Used for 
 * columnar alignment.
 */
extern int
Rule_longestNameLen;

/*
 * Rule_CreateOrdinary
 *
 * Creates an explicit rule.
 */
extern Rule
Rule_CreateOrdinary(char* name,
		    Item original,
		    Item change);

/*
 * Creates a built-in rule (MaximizeOnsets).
 */
extern Rule
Rule_CreateSpecial(Rule_Type type);

/*
 * Rule_Name
 *
 * Returns the name of the rule.
 */
extern char*
Rule_Name(Rule rule);

/*
 * Rule_NameLength
 *
 * Returns the number of characters in the name of the rule.
 */ 
extern int
Rule_NameLength(Rule rule);

/*
 * Rule_Run
 *
 * Runs the rule on the data string, possibly changing it.  onsetPattern
 * is passed in in case the rule is MaximizeOnsets.  If the rule changes the
 * data, the new value is printed, along with the rule name; output format
 * is chosen in accordance with the latex variable.
 */
extern void
Rule_Run(Rule rule, Terminal_String data, Item onsetPattern, Boolean latex);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
