/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/ruleSystem.c
* Description:  Implements ruleSystem.h
* Author:       Brett Kessler
* Created:      Wed Apr 15 16:53:46 1992
* Modified:     Sun May  3 20:10:19 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#include "ruleSystem.h"
#include "header.h"
#include "Malloc.h"
#include "rules.h"
#include "sign.h"
#include "terminal.h"
#include <stdio.h>
#include <stdlib.h>

static RuleSystem
ruleSystem = NULL;

RuleSystem
RuleSystem_Create(Item onsets, Rules rules)
{
  RuleSystem r;
  r = Malloc_Struct(ruleSystem);
  r->onsets = onsets;
  r->rules = rules;
  return r;
}

RuleSystem
RuleSystem_Initialize()
{
  ruleSystem = 
#include "../frag/rules.c"
    ;
  Rules_MeasureLongestName(ruleSystem->rules);
  return ruleSystem;
}

Boolean
RuleSystem_Run(Terminal_String input,
	       Terminal_String expectedOutput,
	       Boolean latex)
{
  static int exampleNumber = 0;
  int c;
  exampleNumber++;
  if (latex) printf("\\begin{tabular}{ll}\n");
  (void) printf("%3d:", exampleNumber);
  if (latex) printf(" &");
  for (c = 3; c < Rule_longestNameLen + 1; c++) putchar(' ');
  Terminal_PrintString(input, stdout, latex); 
  if (latex) printf(" \\\\");
  (void) putchar('\n');
  Rules_Run(ruleSystem->rules, input, ruleSystem->onsets, latex);
  if (latex) {
    printf("\\end{tabular}\n"
	   "\\medskip\n");
  }
  if (Terminal_EqualString(input, expectedOutput)) {
    (void) putchar('\n');
    return TRUE;
  } else {
    (void) printf("***Error: expected ");
    Terminal_PrintString(expectedOutput, stdout, latex);
    (void) puts("\n");
    return FALSE;
  }
}

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
