/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/ruleSystem.h
* Description:  The entire set of rules, as well as the ONSET description.
* Author:       Brett Kessler
* Created:      Wed Apr 15 16:51:19 1992
* Modified:     Sun May  3 20:10:20 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(RULESYSTEM_H)
#define RULESYSTEM_H

#include "header.h"
#include "item.h"
#include "rules.h"
#include "terminal.h"

/*
 * RuleSystem
 *
 * The data structure for the RULES information.
 */
typedef struct ruleSystem {
  Item onsets;
  Rules rules;
}* RuleSystem;

/*
 * RuleSystem_Create
 *
 * Creates the RuleSystem from the ONSETS and RULES items.
 */
extern RuleSystem
RuleSystem_Create(Item onsets, Rules rules);

/*
 * RuleSystem_Initialize
 *
 * Evaluates the single C expression defining the rule system, as was
 * listed in the frag/rules.c file.  Saves the result as a global
 * variable.  Decides which rule name is the longest.
 */
extern RuleSystem
RuleSystem_Initialize(void);

/*
 * RuleSystem_Run
 *
 * Runs all rules against the input string.  Decides whether the final
 * result matches the expectedOutput string, printing an error message
 * and returning FALSE if not.
 */
extern Boolean
RuleSystem_Run(Terminal_String input,
	       Terminal_String expectedOutput,
	       Boolean latex);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
