/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/rules.h
* Description:  All the rules in the RULES section of the RULES declaration.
* Author:       Brett Kessler
* Created:      Wed Apr 15 16:58:36 1992
* Modified:     Sun May  3 20:10:19 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(RULES_H)
#define RULES_H

#include "rule.h"
#include "terminal.h"

/*
 * Rules
 *
 * All the rules, as an array, where the order is significant.
 */
typedef struct rules {
  int nRules;
  Rule* array;
}* Rules;

/*
 * Rules_Create
 *
 * Creates the Rules structure.  The first argument tells how many rules
 * there are; the rest are that many Rule's.
 */
extern Rules
Rules_Create(int nRules, ...);

/*
 * Rules_MeasureLongestName
 *
 * Sets Rule_LongestName to the longest name in the set.
 */
extern void
Rules_MeasureLongestName(Rules rules);

/*
 * Rules_Run
 *
 * Runs all the rules on the input terminal string.  onsetPattern is passed
 * in for the benefit of the MaximizeOnsets special rule.  latex tells what
 * style to print the intermediate forms in.
 */
extern void
Rules_Run(Rules rules, Terminal_String data, Item onsetPattern, Boolean latex);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
