/*  -*- mode:     C -*- --------------------------------------------------
* File:         derive/sign.h
* Description:  A feature sign: +/- or an alpha variable.
* Author:       Brett Kessler
* Created:      Wed Apr 15 17:03:11 1992
* Modified:     Sun May  3 20:45:05 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(SIGN_H)
#define SIGN_H

#include "features.h"
#include "header.h"

typedef enum sign_Variable {
  Sign_ALPHA, Sign_BETA, Sign_GAMMA, Sign_DELTA
} Sign_Variable;

#define Sign_NSIGNS 4  /* actually, the number of different variables.
			  Ought to change this name. */

typedef enum sign_Direction {
  Sign_NO_DIRECTION,  /* never assigned. */
  Sign_PLUS,          /*  +  */
  Sign_MINUS          /*  -  */
} Sign_Direction;

typedef enum sign_Type {
  Sign_constantType,  /* +/- */
  Sign_variableType   /* alpha ... */
} Sign_Type;

/*
 * Sign
 *
 * A value of a feature in a rule.  This could be a +/- constant or an
 * alpha variable.
 */
typedef struct sign {
  Sign_Type type;
  union {
    Sign_Direction direction;  /* Sign_constantType */
    Sign_Variable variable;    /* Sign_variableType */
  } u;
}* Sign;

/*
 * Sign_CreateConstant
 * 
 * Creates a Sign that is a constant (+/-)
 */
extern Sign
Sign_CreateConstant(Sign_Direction direction);

/*
 * Sign_CreateVariable
 *
 * Creates a sign that is a variable (alpha,...)
 */
extern Sign
Sign_CreateVariable(Sign_Variable var);

/*
 * Sign_FeatVector
 * 
 * Saves the +/- values associated with each of the features.  Used
 * when a variable appears on a composite feature, and we need to remember
 * the values of each child feature.
 */
typedef Sign_Direction
Sign_FeatVector[Features_COUNT];

/*
 * Sign_VarFactsType
 *
 * When a variable is encountered, this remembers whether it has never
 * been encountered before in this rule; whether it was on a base feature
 * (therefore representing + or -); or whether it was found on a composite
 * feature.
 */
typedef enum Sign_varFactsType {
  Sign_unsetVarFactsType,
  Sign_directionVarFactsType,
  Sign_compositeVarFactsType
} Sign_VarFactsType;

/*
 * Sign_VarFacts
 *
 * Remembers the facts about when this variable was last encountered.
 */
typedef struct Sign_varFacts {
  Sign_VarFactsType type;
  union {
    Sign_Direction direction; /* Sign_directionVarFactsType */
    Sign_FeatVector signs; /* Sign_compositeVarFactsType */
  } u;
}* Sign_VarFacts;

/* 
 * Sign_VarsVector
 *
 * Remembers the facts about when each of the possible variables was 
 * previously encountered in a rule.
 */
typedef struct Sign_varFacts 
Sign_VarsVector[Sign_NSIGNS];

/*
 * Sign_InitVarsVector
 * 
 * Initializes a vars vector to Sign_unsetVarFactsType for each variable.
 */
extern void
Sign_InitVarsVector(Sign_VarsVector vector);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
