/*  -*- mode:     C -*- --------------------------------------------------
* File:         features/avm.h
* Description:  Builds AVM from FEATURES, prints C files.
* Author:       Brett Kessler
* Created:      Sat Mar 28 18:47:06 1992
* Modified:     Sat May  2 14:30:20 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(AVM_H)
#define AVM_H

#include "list.h"
#include <stdio.h>

/*
 * AVM
 *
 * An AVM is a list of feature-value pairs.
 */
typedef struct avm {
  List fvPairs;
}* AVM;

/*
 * AVM_Create
 *
 * Creates an AVM from a list of feature-value pairs.
 */
extern AVM
AVM_Create(List fvPairs);

/*
 * AVM_Print
 *
 * Print out the structure in AVM format (for debugging).
 * indent should be 0; it is incremented in recursive calls, for handling
 *  embedded AVMs.
 */
extern void
AVM_Print(AVM avm, FILE* file, int indent);

/*
 * AVM_TransduceToPhonesReader
 *
 * Prints out the contents of the features AVMin the source file format
 *  of other languages.
 * lexFile will get a list of Lex expressions matching the name of a 
 *  feature and returning the number of the feature (counting from 0 in
 *  the order listed in the input file (FEATURES)); see 
 *  sandhi/frag/featuresLex.
 * featureIncludeFile will be C source code suitable for an include file
 *  (*.h) defining Features_COUNT; see sandhi/frag/featuresH.
 * featureCFile will be C source code suitable for an implementation
 *  module (*.c).  This includes Features_Names, Features_parent, and
 *  Features_mask.  See sandhi/frag/featuresC for the output, and
 *  sandhi/derive/features.h for an explanation.
 */
extern void
AVM_TransduceToPhonesReader(AVM avm, 
			    FILE* lexFile,
			    FILE* featureIncludeFile,
			    FILE* featureCFile);

#endif  /* end of code */

/*
******************************************************************************
* Revisions:
* RCS $Log: avm.h,v $
* RCS Revision 1.1  2000/02/29 11:06:23  bkessler
* RCS Backup
* RCS
******************************************************************************

******************************************************************************
*                                End of file
******************************************************************************
*/
