%{

/* features/readFeaturesYacc.y
 *
 * Reads the FEATURES file, generating Lex and C source code files in
 * frag/ for the use of later programmes.  Works by building up a single
 * AVM structure representing the file, then transducing that representation.
 */

#include "avm.h"
#include "readFeaturesYacc.h"
#include "featName.h"
#include "fvPair.h"
#include "list.h"
#include "valueType.h"
#include <stdio.h>

FILE* phonesLex;
FILE* featureInclude;
FILE* featureC;

%}

%union {
  int integer;
  ValueType valueType;
  FeatName featName;
  FvPair fvPair;
  List list;
  AVM avm;
}

%token <integer> BINARY
%token <integer> END_AVM
%token <integer> FEAT_TYPE_PAIR_SEP
%token <integer> FEAT_TYPE_SEP
%token <integer> FEATURES
%token <integer> IDENTIFIER
%token <integer> START_AVM

%type <avm>       avm
%type <featName>  featName
%type <fvPair>    fvPair
%type <list>      fvPairs
%type <valueType> valueType

%%

file:                FEATURES avm
                     {
		       AVM_TransduceToPhonesReader($2, 
						   phonesLex,
						   featureInclude,
						   featureC);
		     }
                   ;

avm:                 START_AVM fvPairs END_AVM
                     { $$ = AVM_Create($2); }
                   ;

fvPairs:             fvPair
                     { $$ = List_Add(NULL, $1); }
                   | fvPairs FEAT_TYPE_PAIR_SEP fvPair
                     { $$ = List_Add($1, $3); }
                   ;

fvPair:              featName FEAT_TYPE_SEP valueType
                     { $$ = FvPair_Create($1, $3); }
                   ;

featName:            IDENTIFIER
                     { $$ = FeatName_Create(yytext); }
                   ;

valueType:           BINARY
                     { $$ = ValueType_CreateBinary(); }
                   | avm
                     { $$ = ValueType_CreateAVM($1); }
                   ;

%%

#include "readFeaturesLex.c"

int
ReadFeaturesYacc_Position(void)
{
  return (int) ftell(yyin);
}

int yywrap()
{
        return 1;
} 

int yyerror(char* s)
{
  int position;
  position = ReadFeaturesYacc_Position();
  fprintf(stderr, "%s in file", s);
  if (position >= 0) fprintf(stderr, ", near position %d\n", position);
  else putchar('\n');
  exit(1);
}

void
ReadFeaturesYacc_Parse(FILE* phonesLexFile,
		       FILE* featureIncludeFile,
		       FILE* featureCFile)
{
  phonesLex = phonesLexFile;
  featureInclude = featureIncludeFile;
  featureC = featureCFile;
  yyparse();
}
