/*  -*- mode:     C -*- --------------------------------------------------
* File:         features/valueType.h
* Description:  Possible feature values: binary or sub-AVM.
* Author:       Brett Kessler
* Created:      Sat Mar 28 18:11:45 1992
* Modified:     Sat May  2 14:36:01 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(VALUETYPE_H)
#define VALUETYPE_H

#include "avm.h"
#include <stdio.h>

typedef enum valueType_Type { ValueType_binary, ValueType_avm} 
ValueType_Type;

/*
 * ValueType
 *
 * Possible values a given feature may take (see fvPair.h).
 * ValueType_binary features implicitly take + or -
 * ValueType_avm features are composite features like Place, taking
 *  AVMs of feature-value pairs as their values.
 */
typedef struct valueType {
  ValueType_Type type;
  AVM avm;   /* only if ValueType_avm, else NULL */
}* ValueType;

/*
 * ValueType_CreateBinary
 *
 * Creates an object specifying binary feature values.
 */
extern ValueType
ValueType_CreateBinary(void);

/*
 * ValueType_CreateAVM
 *
 * Creates an object specifying the given set of feature-value pairs as a
 * value (for some higher-level composite feature).
 */
extern ValueType
ValueType_CreateAVM(AVM avm);

/*
 * ValueType_Print
 *
 * Prints the value in inpur format, for debugging.
 */
extern void
ValueType_Print(ValueType vt, FILE* file, int indent);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
