/*  -*- mode: C -*- -----------------------------------------
 * File:           Resource.c
 * Decsription:    Resource where all elements are freed at once.
 * Author:         Brett Kessler
 * Created:        10-Jan-92
 * Language:       C
 **************************************************************
 */

#include "resource.h"
#include "Malloc.h"
#include <stdlib.h>

Resource
Resource_Create(unsigned int charsPerObject, unsigned int objectsPerBlock)
{
  Resource resource = Malloc(sizeof (struct resource));
  resource->lastBlockInUse = NULL;
  resource->blocks = NULL;
  resource->charsPerObject = charsPerObject;
  if (objectsPerBlock <= 0)
    objectsPerBlock = Resource_DEFAULT_OBJECTS_PER_BLOCK;
  resource->charsPerBlock = objectsPerBlock * charsPerObject;
  resource->nextByte = 0;
  return resource;
}

static void
MallocBlock(Resource resource, ResourceBlock prior)
{
  ResourceBlock block;
  block = Malloc(sizeof (struct resourceBlock));
  block->next = NULL;
  block->objects = Malloc(resource->charsPerBlock);
  if (prior == NULL)
    resource->blocks = block;
  else prior->next = block;
  resource->lastBlockInUse = block;
}

static void
GetBlock(Resource resource)
{
  if (resource->lastBlockInUse == NULL ||
      resource->lastBlockInUse->next == NULL)
    MallocBlock(resource, resource->lastBlockInUse);
  else resource->lastBlockInUse = resource->lastBlockInUse->next;
  resource->nextByte = 0;
}

void*
Resource_Get(Resource resource)
{
  ResourceBlock block;
  void* object;
  if (resource->lastBlockInUse == NULL ||
      resource->nextByte >= resource->charsPerBlock)
    GetBlock(resource);  /* this updates lastBlockInUse, nextByte */
  block = resource->lastBlockInUse;
  object = ((char*) block->objects) + resource->nextByte;
  resource->nextByte += resource->charsPerObject;
  return object;
}

void
Resource_Free(Resource resource)
{
  resource->lastBlockInUse = resource->blocks;
  resource->nextByte = 0;
}
