/*  -*- mode:     C -*- --------------------------------------------------
* File:         rules/feature.c
* Description:  Implements feature.h
* Author:       Brett Kessler
* Created:      Tue Apr 14 18:09:09 1992
* Modified:     Sun May  3 15:14:02 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#include "feature.h"
#include "list.h"
#include "Malloc.h"
#include "plainSignedFeature.h"
#include "print.h"

Feature
Feature_CreateFromPlain(PlainSignedFeature plain)
{
  Feature f;
  f = Malloc_Struct(feature);
  f->alternatives = List_Add(NULL, plain);
  return f;
}

Feature
Feature_CreateFromAlternatives(List /* of PlainSignedFeature */ alternatives)
{
  Feature f;
  f = Malloc_Struct(feature);
  f->alternatives = alternatives;
  return f;
}

/*
 * Note that in "derive", unlike here, a feature is created not from a List
 * of plain signed features, but directly from a variable-length list of
 * plain signed feature arguments.  The first argument tells how many
 * plain signed features there will be.
 */
void
Feature_Print(Feature feature, FILE* file)
{
  Cons psfCons;
  PlainSignedFeature psf;
  int count = 0;
  Print_Indent(file);
  Print_level++;
  (void) fprintf(file, "Feature_Create(%d", feature->alternatives->count);
  List_ForEach(feature->alternatives, psfCons, psf) {
    (void) fprintf(file, ",\n");
    Print_Indent(file);
    (void) fprintf(file, "/* Feature alternative #%d */\n", count);
    PlainSignedFeature_Print(psf, file);
    count++;
  }
  (void) fprintf(file, ")");
  Print_level--;
}

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
