/*  -*- mode:     C -*- --------------------------------------------------
* File:         rules/feature.h
* Description:  A feature bundle element: a list of alternatives
* Author:       Brett Kessler
* Created:      Tue Apr 14 17:23:09 1992
* Modified:     Sun May  3 16:08:29 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(FEATURE_H)
#define FEATURE_H

#include "list.h"
#include "plainSignedFeature.h"
#include <stdio.h>

/*
 * Feature
 *
 * The consituents of an AVM which represents a feature bundle.  This
 * is usually the same as a plain signed feature (i.e., when the list
 * has one element), but sometimes the list has more than one such
 * element, in which case they are alternatives.  For example, in
 * [+syll {+back, -low}], +syll would be one Feature and {+back, -low}
 * would be another.
 */
typedef struct feature {
  List /* of plainSignedFeature */ alternatives;
}* Feature;

/*
 * Feature_CreateFromPlain
 *
 * Creates a Feature from one plain signed feature (alternatives list
 * will have 1 element).
 */
extern Feature
Feature_CreateFromPlain(PlainSignedFeature plain);

/*
 * Feature_CreateFromAlternatives
 *
 * Creates a Feature from a list of alternative plain signed features.
 */
extern Feature
Feature_CreateFromAlternatives(List /* of PlainSignedFeature */ alternatives);

/*
 * Feature_Print
 *
 * Prints on the stream (rules.c) the C code for rebuilding the Feature
 * (in "derive").
 */
extern void
Feature_Print(Feature feature, FILE* file);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
