/*  -*- mode:     C -*- --------------------------------------------------
* File:         rules/item.c
* Description:  Implements item.h
* Author:       Brett Kessler
* Created:      Tue Apr 14 18:01:33 1992
* Modified:     Sun May  3 15:14:00 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#include "item.h"
#include "factor.h"
#include "list.h"
#include "Malloc.h"
#include "print.h"
#include <stdio.h>

Item
Item_Create(Factor firstFactor)
{
  Item item;
  item = Malloc_Struct(item);
  item->factors = List_Create();
  (void) List_Add(item->factors, firstFactor);
  return item;
}

Item
Item_Add(Item item, Factor new)
{
  (void) List_Add(item->factors, new);
  return item;
}

/*
 * Prints the Factors in an Item, as a list of arguments: X, Y, ..., Z.
 * The receiving programme will handle the factors not as a list, but
 * will read a variable number of arguments from its arg list.
 */
static void
PrintFactors(List factors, FILE* file, Item focus)
{
  int count = 0;
  Cons factorCons;
  Factor factor;
  Print_level++;
  List_ForEach(factors, factorCons, factor) {
    (void) fprintf(file, ",\n");
    Print_Indent(file);
    (void) fprintf(file, "/* Item #%d */\n", count);
    Factor_Print(factor, file, focus);
    count++;
  }
  Print_level--;
}

void
Item_Print(Item item, FILE* file, Item focus)
{
  Print_Indent(file);
  if (item == NULL)
    (void) fprintf(file, "NULL");
  else {
    (void) fprintf(file, "Item_Create(%d", item->factors->count);
    PrintFactors(item->factors, file, focus);
    (void) fprintf(file, ")");
  }
}

void
Item_PrintOnsets(Item onsets, FILE* file)
{
  (void) fprintf(file, "Item_Create(%d", onsets->factors->count + 2);
  (void) fprintf(file, ",\n");
  Print_Indent(file);
  (void) fprintf(file, "Factor_CreateFromBoundary(Factor_syllableBoundary)");
  PrintFactors(onsets->factors, file, NULL);
  (void) fprintf(file, ",\n");
  Print_Indent(file);
  (void) fprintf(file, "Factor_CreateFromBoundary(Factor_utteranceBoundary)");
  (void) fprintf(file, ")");
}

/* There's no need to return distinctive Items here, so we keep returning
 * a pointer to the same one each time.
 */
Item
Item_NoEnvironment(void)
{
  static Item item = NULL;
  if (item == NULL) {
    Factor factor;
    factor = Factor_CreateFromEnvironbar();
    item = Item_Create(factor);
  }
  return item;
}

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
