/*  -*- mode:     C -*- --------------------------------------------------
* File:         rules/item.h
* Description:  A sequence of Factor: the top level pattern in rules.
* Author:       Brett Kessler
* Created:      Tue Apr 14 17:14:56 1992
* Modified:     Sun May  3 15:29:53 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(ITEM_H)
#define ITEM_H

#include "factor.h"
#include "list.h"
/*
 * Item
 *
 * The item is a sequence of Factor, for matching a sequence of segments
 * and boundaries.
 */
typedef struct item {
  List /* of Factor*/ factors;
} *Item;

/*
 * Item_Create
 *
 * Creates an Item and initializes it with its first constituent Factor.
 */
extern Item
Item_Create(Factor firstFactor);

/*
 * Item_Add
 *
 * Adds a new Factor at the end of the Item's list of Factors, and returns
 * the same Item.
 */
extern Item
Item_Add(Item item, Factor new);

/*
 * Item_Print
 *
 * Prints on the stream (frag/rules.c) the C code necessary for rebuilding
 * the Item structure in another programme ("derive").  The focus argument
 * is the source description, necessary because it is substututed for an
 * environment bar, thus creating a sequential structural description.
 */
extern void
Item_Print(Item item, FILE* file, Item focus);

/*
 * Item_PrintOnsets
 *
 * Prints on the stream the C code for rebuilding the ONSETS description.
 * This actually wraps a SB and UB around the description to ensure
 * that it matches the entire onset (which will be similarly delimited).
 */
extern void
Item_PrintOnsets(Item onsets, FILE* file);

/*
 * Item_NoEnvironment
 *
 * Returns an item that consists of an environment bar factor.  This is
 * used in normalizing rules that don't have environments: thus, the
 * rule  A -> B  can be treated as  A -> B / __
 */
extern Item
Item_NoEnvironment(void);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
