/*  -*- mode:     C -*- --------------------------------------------------
* File:         rules/plainSignedFeature.h
* Description:  A phonetic feature with a sign or alpha variable.
* Author:       Brett Kessler
* Created:      Thu Apr  9 09:01:21 1992
* Modified:     Sun May  3 16:08:28 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(PLAINSIGNEDFEATURE_H)
#define PLAINSIGNEDFEATURE_H

#include "list.h"
#include "sign.h"
#include <stdio.h>

/*
 * PlainSignedFeature
 *
 * A phonetic feature with a sign, which may be either a constant (+ or -)
 * or a variable (alpha, beta...). The feature itself is represented by its
 * arbitrary numeric ID.
 */
typedef struct plainSignedFeature {
  Sign sign;
  int featureNumber;
}* PlainSignedFeature;

/*
 * PlainSignedFeature_Create 
 * 
 * Creates a plain signed feature from a sign and feature ID number.
 */
extern PlainSignedFeature
PlainSignedFeature_Create(Sign sign, int featureNumber);

/*
 * PlainSignedFeature_Print
 *
 * Prints on the stream (frag/rules.c) the C code for rebuilding the 
 * object in another programme ("derive").
 */
extern void
PlainSignedFeature_Print(PlainSignedFeature feat, FILE* file);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
