/*  -*- mode:     C -*- --------------------------------------------------
* File:         rules/rule.h
* Description:  One phonological rule.
* Author:       Brett Kessler
* Created:      Thu Apr  9 09:04:01 1992
* Modified:     Sun May  3 16:08:27 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(RULE_H)
#define RULE_H

#include "item.h"

/*
 * Rule_Type
 *
 * Most rules are ordinary, explicit rules.  The MaximizeOnsets rule is
 * a special built-in rule for resyllabifying the string in accordance 
 * with the maximum onsets principle, using the ONSETS declaration.
 */
typedef enum rule_Type {
  Rule_ordinaryType,
  Rule_maximizeOnsets
} Rule_Type;

/*
 * Rule
 *
 * A rule.  If the type is Rule_ordinaryType, the other fields will have
 * meaningful values.  The three Items are the principle components of
 * the rule:   original -> change / environment
 */
typedef struct rule {
  Rule_Type type;
  char* name;
  Item original;
  Item change;
  Item environment;
}* Rule;

/*
 * Rule_CreateOrdinary
 *
 * Creates an ordinary, explicit rule.
 */
extern Rule
Rule_CreateOrdinary(char* name,
		    Item original,
		    Item change,
		    Item environment);

/*
 * Rule_CreateSpecial
 *
 * Creates a built-in rule (MaximizeOnsets).
 */
extern Rule
Rule_CreateSpecial(Rule_Type specialRuleNumber);

/*
 * Rule_Print
 *
 * Prints on the stream (frag/rules.c) C code for rebuilding the Rule
 * object in another programme ("derive").
 */
extern void
Rule_Print(Rule rule, FILE* file);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
