/*  -*- mode:     C -*- --------------------------------------------------
* File:         rules/sign.h
* Description:  A sign in front of a feature: constant +/- or alpha var.
* Author:       Brett Kessler
* Created:      Thu Apr  9 09:14:42 1992
* Modified:     Sun May  3 16:12:28 1992 (Brett Kessler)
* Language:     C
******************************************************************************
*/

#if !defined(SIGN_H)
#define SIGN_H

#include <stdio.h>

typedef enum sign_Variable {
  Sign_ALPHA, Sign_BETA, Sign_GAMMA, Sign_DELTA
} Sign_Variable;

typedef enum sign_Direction {
  Sign_PLUS, /*  +  */
  Sign_MINUS /*  -  */
} Sign_Direction;

typedef enum sign_Type {
  Sign_constantType,   /* takes a Sign_Direction */
  Sign_variableType    /* takes a Sign_Variable */
} Sign_Type;

/*
 * Sign
 *
 * A Sign is either a constant (+/-) or a variable (alpha, beta, ...).
 */
typedef struct sign {
  Sign_Type type;
  union {
    Sign_Direction direction;  /* Sign_constantType */
    Sign_Variable variable;    /* Sign_variableType */
  } u;
}* Sign;

/*
 * Sign_CreateConstant
 *
 * Creates a constant (+/-) sign.
 */
extern Sign
Sign_CreateConstant(Sign_Direction direction);

/*
 * Sign_CreateVariable
 *
 * Creates a variable (alpha, beta...) standing in for a sign.
 */
extern Sign
Sign_CreateVariable(Sign_Variable sign);

/*
 * Sign_Print
 *
 * Writes on the stream (frag/rules.c) the C code necessary for rebuilding
 * the Sign structure in another programme ("derive").
 */
extern void
Sign_Print(Sign sign, FILE* file);

#endif  /* end of code */

/*
******************************************************************************
*                                End of file
******************************************************************************
*/
